
# q-expansion of newform 171.5.c.c, downloaded from the LMFDB on 21 September 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 171
weight = 5
poly_data = [142, 0, 35, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[19, 0, 1, 0], 3], [[0, 28, 0, 1], 3]]

hecke_ring_character_values = [[20, [1, 0, 0, 0]], [154, [-1, 0, 0, 0]]]
aps_data = [[0, 1, 0, 0], [0, 0, 0, 0], [10, 0, 1, 0], [31, 0, 6, 0], [-64, 0, 17, 0], [0, 10, 0, -11], [-63, 0, -24, 0], [0, 38, 57, -19], [-37, 0, 113, 0], [0, -208, 0, -3], [0, 242, 0, 56], [0, -282, 0, 72], [0, 430, 0, -36], [922, 0, -351, 0], [1852, 0, 85, 0], [0, 204, 0, 63], [0, -250, 0, 303], [-46, 0, 171, 0], [0, 428, 0, -151], [0, -516, 0, -390], [-3963, 0, 342, 0], [0, 566, 0, -76], [-8188, 0, -262, 0], [0, 1374, 0, -726], [0, 640, 0, 70], [4498, 0, -2420, 0], [0, 2860, 0, -860], [0, 3714, 0, -93], [0, 1004, 0, 227], [0, -910, 0, -1092], [0, -4458, 0, -330], [-3110, 0, -5573, 0], [15071, 0, -4642, 0], [2660, 0, -3981, 0], [-10868, 0, 4819, 0], [0, -2456, 0, 1894], [10834, 0, -4008, 0], [7026, 0, 2028, 0], [0, 3886, 0, 708], [0, -3688, 0, 1638], [0, 5866, 0, -2136], [0, 12360, 0, 1578], [23479, 0, -6056, 0], [0, -8998, 0, -1156], [-4002, 0, 9456, 0], [13359, 0, 1974, 0], [0, 3334, 0, -5213], [0, -6926, 0, 382], [0, 2078, 0, -2511], [-41638, 0, -501, 0], [20688, 0, -3201, 0], [-20005, 0, 9236, 0], [0, -9190, 0, 2648], [-17688, 0, -15543, 0], [0, 10600, 0, -5148], [-50888, 0, -13031, 0], [0, -14154, 0, -1440], [75567, 0, -3039, 0], [-27476, 0, -819, 0], [0, 10156, 0, 3444], [14518, 0, -23979, 0], [0, -2134, 0, 177], [0, 9078, 0, -5472], [44465, 0, 16730, 0], [-44461, 0, 27705, 0], [0, -3506, 0, -1965], [0, 29708, 0, 4883], [0, -21356, 0, -8612], [-120094, 0, 9275, 0], [142376, 0, 5217, 0], [-139145, 0, 14161, 0], [15141, 0, 22326, 0], [2090, 0, 48516, 0], [0, 26594, 0, -5953], [0, -19854, 0, -3339], [0, 4826, 0, 11190], [83048, 0, 35543, 0], [-97428, 0, 28293, 0], [0, 22654, 0, 7530], [0, 19646, 0, 13916], [51196, 0, 8530, 0], [0, -45032, 0, 15001], [0, -51768, 0, 96], [0, 29346, 0, -1710], [0, 6526, 0, 4078], [102232, 0, 23569, 0], [0, -41776, 0, 17310], [61153, 0, 14634, 0], [-209414, 0, 36529, 0], [98236, 0, -26565, 0], [-46784, 0, -74471, 0], [74306, 0, 41432, 0], [0, 36336, 0, 19824], [-263996, 0, -16742, 0], [-44758, 0, -25551, 0], [398043, 0, -11871, 0], [0, -18028, 0, 25506], [0, -11950, 0, 24456], [0, -8862, 0, -27411], [454626, 0, -11661, 0], [0, 37136, 0, -4522], [163730, 0, 9473, 0], [0, 44866, 0, -38616], [0, -22928, 0, 25314], [-299686, 0, -19068, 0], [-210117, 0, -34548, 0], [-177070, 0, -19285, 0], [-79734, 0, 4932, 0], [0, 49334, 0, -15234], [0, -62860, 0, 1466], [0, -46204, 0, -23578], [-179190, 0, 41955, 0], [43264, 0, 77935, 0], [-76012, 0, 7374, 0], [123000, 0, -133833, 0], [0, -61472, 0, 1908], [-2260, 0, -53277, 0], [-196335, 0, -116574, 0], [609512, 0, 19703, 0], [0, 66082, 0, 39885], [0, -87566, 0, 36607], [0, -5432, 0, 31210], [0, 5056, 0, -8925], [0, 56340, 0, 13002], [442298, 0, 46809, 0], [428040, 0, -29358, 0], [322288, 0, 29094, 0], [-696225, 0, 3192, 0], [-78109, 0, 18786, 0], [-299732, 0, 68514, 0], [101810, 0, 81081, 0], [0, -50810, 0, 6264], [0, 181588, 0, -19670], [-331650, 0, 7647, 0], [-695293, 0, 77066, 0], [523591, 0, 59040, 0], [0, 89134, 0, 23361], [0, -193322, 0, -20465], [0, 142912, 0, -62331], [-293029, 0, 120794, 0], [0, 203598, 0, -25323], [-26080, 0, -134569, 0], [-720749, 0, 152394, 0], [0, 39192, 0, -1743], [0, -169014, 0, -6519], [0, 77248, 0, -33072], [-729092, 0, -93102, 0], [0, -72736, 0, -804], [867070, 0, -78099, 0], [0, -63642, 0, -73986], [0, 266146, 0, 43633], [571404, 0, 125067, 0], [72558, 0, 144381, 0], [0, 222368, 0, -49284], [0, 91108, 0, -52601], [0, -136324, 0, 44256], [-403671, 0, 100227, 0], [702271, 0, -184487, 0], [218277, 0, -306330, 0], [0, -175786, 0, -55269], [-792452, 0, 27562, 0], [0, -100722, 0, 16590], [315742, 0, -122592, 0], [0, 224640, 0, 60126], [0, -166214, 0, -73428], [0, 78964, 0, 68652], [0, 93282, 0, 27606], [44740, 0, -181947, 0]]
